#!/usr/local/bin/ruby
# by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)

module RGUI
  FontSize = 14
  FontWidth = FontSize / 2
  FontDec = 2
  StatWidth = FontSize * 3

  def initialize(*args)
    super
    @childlen = []
  end

  def add(*args)
    @childlen += args.flatten
  end

  def button_press(e)
    return if e.button != 1
    @childlen.each { |c|
      next unless c.type.method_defined? :_button_press
      next unless c.x < e.x && e.x < c.x + c.width && c.y < e.y && e.y < c.y + c.height
      c._button_press(e)
    }
  end

  def button_release(e)
    return if e.button != 1
    @childlen.each { |c|
      next unless c.type.method_defined? :_button_release
    # next unless c.x < e.x && e.x < c.x + c.width && c.y < e.y && e.y < c.y + c.height
      c._button_release(e)
    }
  end

  def key_press(e)
    @childlen.each { |c|
      next unless c.type.method_defined? :_key_press
      c._key_press(e)
    }
  end

  def key_release(e)
    @childlen.each { |c|
      next unless c.type.method_defined? :_key_release
      c._key_release(e)
    }
  end

  def motion_notify(e)
    @childlen.each { |c|
      next unless c.type.method_defined? :_motion_notify
      c._motion_notify(e)
    }
  end

  def exposed(e)
    repaint
    @childlen.each { |c|
      next unless c.type.method_defined? :_exposed
      c._exposed
    }
  end

  def method_missing(m,*args)
    if m == :repaint then return end
    raise NameError, "unknown method :#{m.id2name} for #{type}"
  end

end  # of module RGUI

require 'rgui/widget.rb'
require 'rgui/window.rb'
require 'rgui/button.rb'
require 'rgui/hbox.rb'
require 'rgui/vbox.rb'
require 'rgui/label.rb'
require 'rgui/frame.rb'
require 'rgui/panel.rb'
require 'rgui/entry.rb'
require 'rgui/text.rb'
require 'rgui/scroll.rb'

