#!/usr/local/bin/ruby
# by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)

module RGUI
  class Scroll < Widget
    def initialize(gc, src, button_gc)
      super(gc)
      @src = src
      @barw = 48
      @barh = 48
      @icow = 16
      @icoh = 16
      @src.scroll = self
      @hscr = Button.new button_gc, 'hscr'
      @vscr = Button.new button_gc, 'v'
      @iw = 0
      @ih = 0
    end
    attr :src
    
    def abs(n)
      if n<0 then -n else n end
    end

    def layout
      @src.x = @x
      @src.y = @y
      @src.width  = @width  - @icow
      @src.height = @height - @icoh
      @src._exposed

      @icox = if @iw == @src.width
        @icow
      else
        @icow + @src.scrx * (@width  - 3*@icow - @barw) / (@iw - @src.width )
      end
      @icox = gt @icox, @icow
      @icox = lt @icox, @width  - 2*@icow - @barw

      @icoy = if @ih == @src.height
        @icoh
      else
        @icoh + @src.scry * (@height - 3*@icoh - @barh) / (@ih - @src.height)
      end
      @icoy = gt @icoy, @icoh
      @icoy = lt @icoy, @height - 2*@icoh - @barh

      b = @hscr
      b.x = @x + @icox
      b.y = @y + @height - @icoh
      b.width  = @barw
      b.height = @icoh

      b = @vscr
      b.x = @x + @width  - @icow
      b.y = @y + @icoy
      b.width  = @icow
      b.height = @barh

      _exposed
    end
    
    def image_width=(iw)
      @iw = iw
      @iw = gt @iw, @src.width
    end

    def image_height=(ih)
      @ih = ih
      @ih = gt @ih, @src.height
    end

    def motion_notify_hscr(m=nil)
      e = @hscr
      if e.drag
        sx = m.x - @dx
        sx = gt(sx, @x + @icow)
        sx = lt(sx, @x + @width - 2*@icow - @barw)
        return if e.x == sx
        e.x = sx
        exposed_hscr
      else
        if e.x < m.x && m.x < e.x + e.width
          unless e.act
            if e.drag || e.y < m.y && m.y < e.y + e.height
              e._motion_notify m
              exposed_hscr
            end
          end
        else
          return unless e.act
          e._motion_notify m
          exposed_hscr
        end
      end
    end

    def motion_notify_vscr(m=nil)
      e = @vscr
      if e.drag
        sy = m.y - @dy
        sy = gt(sy, @y + @icoh)
        sy = lt(sy, @y + @height - 2*@icoh - @barh)
        return if e.y == sy
        e.y = sy
        exposed_vscr
      else
        if e.y < m.y && m.y < e.y + e.height
          unless e.act
            if e.drag || e.x < m.x && m.x < e.x + e.width
              e._motion_notify m
              exposed_vscr
            end
          end
        else
          return unless e.act
          e._motion_notify m
          exposed_vscr
        end
      end
    end

    def _motion_notify(e=nil)
      motion_notify_hscr e
      motion_notify_vscr e
      @src.motion_notify e
    end
    
    def button_press_hscr(m=nil)
      e = @hscr
      return if e.drag
      if e.x < m.x && m.x < e.x + e.width
        @dx = e.width / 2
        e._button_press m
        exposed_hscr
      end
    end

    def button_press_vscr(m=nil)
      e = @vscr
      return if e.drag
      if e.y < m.y && m.y < e.y + e.height
        @dy = e.height / 2
        e._button_press m
        exposed_vscr
      end
    end

    def _button_press(e=nil)
      button_press_hscr e
      button_press_vscr e
      @src.button_press e
    end
    
    def button_release_hscr(m=nil)
      e = @hscr
      return unless e.drag
      e._button_release m
      exposed_vscr
      hscr_moveto e.x
      @dx = nil
    end

    def button_release_vscr(m=nil)
      e = @vscr
      return unless e.drag
      e._button_release m
      exposed_vscr
      vscr_moveto e.y
      @dy = nil
    end

    def _button_release(e=nil)
      button_release_hscr e
      button_release_vscr e
      @src.button_release e
    end
    
    def _key_press(s=nil)
      @src.key_press s
    end

    def _key_release(s=nil)
      @src.key_release s
    end

    def exposed_hscr(e=nil)
      @gc.draw_hscr self
      @hscr._exposed e
    end

    def exposed_vscr(e=nil)
      @gc.draw_vscr self
      @vscr._exposed e
    end

    def _exposed(e=nil)
      exposed_hscr e
      exposed_vscr e
    end
    
    def hscr_moveto(x)
      @icox = x - (@x + @icow)
      @src.scrx = @icox * (@iw - @src.width ) / (@width  - 3*@icow - @barw)
    end

    def vscr_moveto(y)
      @icoy = y - (@y + @icoh)
      @src.scry = @icoy * (@ih - @src.height) / (@height - 3*@icoh - @barh)
    end

  end
end
