#!/usr/local/bin/ruby
# by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)

require 'wide'
require 'xlib/keysym'

module RGUI
  class Text < Widget
    private

    def colmax(col) (col - 3) * FontWidth end
    def colmin(col) (col + 3 - @width  / FontWidth) * FontWidth end
    def rowmax(row) (row - 3) * FontSize  end
    def rowmin(row) (row + 3 - @height / FontSize ) * FontSize  end
    
    def initialize(gc,x=0,y=0,width=80,height=80)
      super(gc)
      @win = @gc.window
      @x,@y,@width,@height = x,y,width,height
      @row = 0
      @col = 0
      @ary = ["\n"]
      @rows = 0
      @cols = 0
      @scrx = 0
      @scry = 0
      exposed
    end
  
    public 

    attr :ary
    attr :row
    attr :col
    attr :rows
    attr :cols
    attr :scrx
    attr :scry
    attr :scroll,true

    def layout
      exposed
    end
  
    def text
      @ary.join("\n").chop
    end
  
    def text=(text)
      @ary = []
      @cols = 0
      text.each("\n") { |line|
        @ary << line
        @cols = gt @cols, line.length
      }
      @rows = @ary.length
      @ary << "\n"
      @col = 0
      @row = 0
      @scrx = 0
      @scry = 0
      repaint
    end
  
    def button_press(e)
      @win.focus_widget = self
    end
  
    def key_press(e)
      return unless @focus
      @gc.draw_cursor self,:hide
      need_repaint = false
      if e.string
        s = e.string
        @ary[@row][@col,1] = s+@ary[@row][@col,1]
        @col += s.length
        need_repaint = true
      else
        case e.keysym
        when 0x20..0xff
          s = e.keysym.chr
          @ary[@row][@col,1] = s+@ary[@row][@col,1]
          @col += s.length
          need_repaint = true
        when Xlib::Keysym.id('Return')
          if @col == 0
            @ary[@row,1] = ["\n"]+@ary[@row]
          else
            @ary[@row,1] = [@ary[@row][0..(@col-1)]+"\n",@ary[@row][@col..-1]]
          end
          @col = 0
          @row+= 1
          need_repaint = true
        when Xlib::Keysym.id('BackSpace')
          if @col == 0
            if @row == 0
            else
              @row-= 1
              @col = @ary[@row].length-1
              @ary[@row,2] = @ary[@row].chop+@ary[@row+1]
            end
          else
            @col-= 1
            wide = Wide::wide_at(@ary[@row], @col)
            @col-= wide
            @ary[@row][@col,wide+1] = ""
          end
          need_repaint = true
        when Xlib::Keysym.id('Left')
          if @col == 0
            if @row == 0
            else
              @row-= 1
              @col = @ary[@row].length-1
            end
          else
            @col-= 1
            @col-= Wide::wide_at(@ary[@row], @col)
          end
        when Xlib::Keysym.id('Up')
          if @row == 0
          else
            @row-= 1
            @col = lt @col,@ary[@row].length-1
            @col-= Wide::wide_at(@ary[@row], @col)
          end
        when Xlib::Keysym.id('Right')
          if @col == @ary[@row].length-1
            if @row == @ary.length-1
            else
              @row+= 1
              @col = 0
            end
          else
            @col+= 1
            @col+= Wide::wide_at(@ary[@row], @col)
          end
        when Xlib::Keysym.id('Down')
          if @row == @ary.length-1
          else
            @row+= 1
            @col = lt @col, @ary[@row].length-1
            @col-= Wide::wide_at(@ary[@row], @col)
          end
        when Xlib::Keysym.id('Shift_L')
        when Xlib::Keysym.id('Shift_R')
        when Xlib::Keysym.id('Control_L')
        when Xlib::Keysym.id('Control_R')
        else
          print "#{type} unknown keysym: #{e.keysym} '#{Xlib::Keysym.name e.keysym}'\n"
        end
      end
      @rows = gt @rows,@ary.length-1
      @cols = gt @cols,@ary[@row].length-1
      x = @scrx
      x = lt x,colmax(@col)
      x = gt x,colmin(@col)
      x = gt x,0
      if @scrx != x then @scrx = x; need_repaint = true; end
      y = @scry
      y = lt y,rowmax(@row)
      y = gt y,rowmin(@row)
      y = gt y,0
      if @scry != y then @scry = y; need_repaint = true; end
      if need_repaint
        if @scroll then @scroll.layout else repaint end
      else
        @gc.draw_cursor self,:show
      end
    end
    
    def scrx=(x)
      if @scrx != x then @scrx = x; repaint; end
    end
  
    def scry=(y)
      if @scry != y then @scry = y; repaint; end
    end
  
    def set_im_area
      begin
        @win.im.edit = Xlib::Point.new @x + FontWidth * @col - @scrx, @y + FontSize * (@row+1) - @scry
        @win.im.stat = Xlib::Rect .new @x + FontWidth * @col - @scrx, @y + FontSize * (@row+1) + FontDec - @scry, StatWidth, FontSize
      rescue
        p $!
      end
    end
  
    def exposed(e=nil)
      @pix = @win.new_pixmap @width,@height
      @gc.window = @pix
      repaint
    end
  
    def repaint
      @gc.repaint self
      @pix.copy_area @win,@gc,@width,@height,@x,@y
      @gc.draw_cursor self,:show
      set_im_area
      @scroll.image_width  = (@cols+3) * FontWidth if @scroll
      @scroll.image_height = (@rows+3) * FontSize  if @scroll
    end
  end
end
